@@ -28,6 +28,7 @@ from tacker.sol_refactored.common import vnf_instance_utils as inst_utils
 from tacker.sol_refactored.infra_drivers.kubernetes import helm
 from tacker.sol_refactored.infra_drivers.kubernetes import kubernetes
 from tacker.sol_refactored.infra_drivers.openstack import openstack
+from tacker.sol_refactored.infra_drivers.terraform import terraform
 from tacker.sol_refactored.nfvo import nfvo_client
 from tacker.sol_refactored import objects
 from tacker.sol_refactored.objects.v2 import fields as v2fields
@@ -94,7 +95,8 @@ class VnfLcmDriverV2(object):
             method(context, lcmocc, inst, grant_req, grant, vnfd)
 
     def _exec_mgmt_driver_script(self, operation, flavour_id, req, inst,
-            grant_req, grant, vnfd):
+                                 grant_req, grant, vnfd,
+                                 user_script_err_handling_data, new_vnfd=None):
         script = vnfd.get_interface_script(flavour_id, operation)
         if script is None:
             return
@@ -108,8 +110,12 @@ class VnfLcmDriverV2(object):
                               if grant_req is not None else None),
             'grant_response': (grant.to_dict()
                                if grant is not None else None),
-            'tmp_csar_dir': tmp_csar_dir
+            'tmp_csar_dir': tmp_csar_dir,
+            'user_script_err_handling_data': user_script_err_handling_data
         }
+        if new_vnfd:
+            new_csar_dir = new_vnfd.make_tmp_csar_dir()
+            script_dict['new_csar_dir'] = new_csar_dir
         # script is relative path to Definitions/xxx.yaml
         script_path = os.path.join(tmp_csar_dir, "Definitions", script)
 
@@ -119,21 +125,25 @@ class VnfLcmDriverV2(object):
 
         vnfd.remove_tmp_csar_dir(tmp_csar_dir)
 
+        try:
+            output = pickle.loads(out.stdout)
+            if isinstance(output, dict):
+                if 'vnf_instance' in output:
+                    _inst = objects.VnfInstanceV2.from_dict(
+                        output['vnf_instance'])
+                    inst.__dict__.update(_inst.__dict__)
+                if 'user_script_err_handling_data' in output:
+                    user_script_err_handling_data.update(output[
+                        'user_script_err_handling_data'])
+        except EOFError:
+            pass
+        except pickle.UnpicklingError:
+            pass
+
         if out.returncode != 0:
             LOG.debug("execute %s failed: %s", operation, out.stderr)
             msg = "{} failed: {}".format(operation, out.stderr)
             raise sol_ex.MgmtDriverExecutionFailed(sol_detail=msg)
-        else:
-            try:
-                output = pickle.loads(out.stdout)
-                if isinstance(output, dict) and 'vnf_instance' in output:
-                    _inst = objects.VnfInstanceV2.from_dict(
-                        output['vnf_instance'])
-                    inst.__dict__.update(_inst.__dict__)
-            except EOFError:
-                pass
-            except pickle.UnpicklingError:
-                pass
 
         LOG.debug("execute %s of %s success.", operation, script)
 
@@ -205,15 +215,18 @@ class VnfLcmDriverV2(object):
     def _script_method_name(self, operation):
         # According to the definition in etsi_nfv_sol001_vnfd_types.yaml,
         # get script method name from lcmocc.operation.
-        # only MODIFY_INFO and CHANGE_EXT_CONN are exceptional.
+        # only MODIFY_INFO, CHANGE_EXT_CONN, and CHANGE_VNFPKG are exceptional.
         if operation == v2fields.LcmOperationType.MODIFY_INFO:
             return "modify_information"
         elif operation == v2fields.LcmOperationType.CHANGE_EXT_CONN:
             return "change_external_connectivity"
+        elif operation == v2fields.LcmOperationType.CHANGE_VNFPKG:
+            return "change_current_package"
         else:
             return operation.lower()
 
-    def process(self, context, lcmocc, inst, grant_req, grant, vnfd):
+    def process(self, context, lcmocc, inst, grant_req, grant, vnfd,
+                user_script_err_handling_data):
         # save inst to use updating lcmocc after process done
         inst_saved = inst.obj_clone()
 
@@ -229,30 +242,60 @@ class VnfLcmDriverV2(object):
             # NOTE: This is only the case that operation is MODIFY_INFO
             # and vnf instance is never instantiated after creation.
             flavour_id = None
-        self._exec_mgmt_driver_script(operation,
-            flavour_id, req, inst, grant_req, grant, vnfd)
+        self._exec_mgmt_driver_script(
+            operation, flavour_id, req, inst, grant_req, grant, vnfd,
+            user_script_err_handling_data)
 
         # main process
         method = getattr(self, "%s_%s" % (lcmocc.operation.lower(), 'process'))
         method(context, lcmocc, inst, grant_req, grant, vnfd)
 
         # perform postamble LCM script
+        new_vnfd = None
+        if (lcmocc.operation == v2fields.LcmOperationType.MODIFY_INFO
+                and req.obj_attr_is_set('vnfdId')):
+            new_vnfd = self.nfvo_client.get_vnfd(
+                context, req.vnfdId, all_contents=True)
         operation = "{}_end".format(self._script_method_name(lcmocc.operation))
-        self._exec_mgmt_driver_script(operation,
-            flavour_id, req, inst, grant_req, grant, vnfd)
+        self._exec_mgmt_driver_script(
+            operation, flavour_id, req, inst, grant_req, grant, vnfd,
+            user_script_err_handling_data, new_vnfd=new_vnfd)
 
         self._make_inst_info_common(lcmocc, inst_saved, inst, vnfd)
         lcmocc_utils.update_lcmocc(lcmocc, inst_saved, inst)
 
-    def rollback(self, context, lcmocc, inst, grant_req, grant, vnfd):
+    def rollback(self, context, lcmocc, inst, grant_req, grant, vnfd,
+                 user_script_err_handling_data):
         method = getattr(self,
                          "%s_%s" % (lcmocc.operation.lower(), 'rollback'),
                          None)
-        if method:
-            method(context, lcmocc, inst, grant_req, grant, vnfd)
-        else:
+        if not method:
             raise sol_ex.RollbackNotSupported(op=lcmocc.operation)
 
+        # perform preamble LCM script
+        req = lcmocc.operationParams
+        operation_name = self._script_method_name(lcmocc.operation)
+        operation = f"{operation_name}_rollback_start"
+        if lcmocc.operation == v2fields.LcmOperationType.INSTANTIATE:
+            flavour_id = req.flavourId
+        elif inst.obj_attr_is_set('instantiatedVnfInfo'):
+            flavour_id = inst.instantiatedVnfInfo.flavourId
+        else:
+            # NOTE: This is only the case that operation is MODIFY_INFO
+            # and vnf instance is never instantiated after creation.
+            flavour_id = None
+        self._exec_mgmt_driver_script(
+            operation, flavour_id, req, inst, grant_req,
+            grant, vnfd, user_script_err_handling_data)
+
+        method(context, lcmocc, inst, grant_req, grant, vnfd)
+
+        # perform postamble LCM script
+        operation = f"{operation_name}_rollback_end"
+        self._exec_mgmt_driver_script(
+            operation, flavour_id, req, inst, grant_req,
+            grant, vnfd, user_script_err_handling_data)
+
     def _get_link_ports(self, inst_req):
         names = set()
         if inst_req.obj_attr_is_set('extVirtualLinks'):
@@ -344,7 +387,7 @@ class VnfLcmDriverV2(object):
             grant_req.addResources = add_reses
 
         # placementConstraints
-        self._make_placementconstraints(grant_req, vnfd, add_reses)
+        self._make_placementconstraints(grant_req, vnfd, add_reses, inst)
 
         if req.obj_attr_is_set('additionalParams'):
             grant_req.additionalParams = req.additionalParams
@@ -426,6 +469,9 @@ class VnfLcmDriverV2(object):
         elif vim_info.vimType == 'ETSINFV.HELM.V_3':
             driver = helm.Helm()
             driver.instantiate(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'TERRAFORM.V1':
+            driver = terraform.Terraform()
+            driver.instantiate(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
@@ -445,6 +491,9 @@ class VnfLcmDriverV2(object):
         elif vim_info.vimType == 'ETSINFV.HELM.V_3':
             driver = helm.Helm()
             driver.instantiate_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'TERRAFORM.V1':
+            driver = terraform.Terraform()
+            driver.instantiate_rollback(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
@@ -590,6 +639,9 @@ class VnfLcmDriverV2(object):
         elif vim_info.vimType == 'ETSINFV.HELM.V_3':
             driver = helm.Helm()
             driver.terminate(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'TERRAFORM.V1':
+            driver = terraform.Terraform()
+            driver.terminate(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
@@ -672,9 +724,9 @@ class VnfLcmDriverV2(object):
             grant_req.addResources = add_reses
 
         # placementConstraints
-        self._make_placementconstraints(grant_req, vnfd, add_reses)
+        self._make_placementconstraints(grant_req, vnfd, add_reses, inst)
 
-    def _make_placementconstraints(self, grant_req, vnfd, add_reses):
+    def _make_placementconstraints(self, grant_req, vnfd, add_reses, inst):
         affinity_policies = {
             'AFFINITY': vnfd.get_affinity_targets(grant_req.flavourId),
             'ANTI_AFFINITY': vnfd.get_anti_affinity_targets(
@@ -684,14 +736,34 @@ class VnfLcmDriverV2(object):
         for key, value in affinity_policies.items():
             for targets, scope in value:
                 res_refs = []
-                for target in targets:
-                    for res in add_reses:
-                        if res.resourceTemplateId == target:
+                for res in add_reses:
+                    if res.resourceTemplateId in targets:
+                        res_ref = objects.ConstraintResourceRefV1(
+                            idType='GRANT',
+                            resourceId=res.id)
+                        res_refs.append(res_ref)
+
+                if not res_refs:
+                    break
+
+                if (inst.obj_attr_is_set('instantiatedVnfInfo') and
+                        inst.instantiatedVnfInfo.obj_attr_is_set(
+                            'vnfcResourceInfo')):
+                    for vnfc in inst.instantiatedVnfInfo.vnfcResourceInfo:
+                        if vnfc.vduId in targets:
                             res_ref = objects.ConstraintResourceRefV1(
-                                idType='GRANT',
-                                resourceId=res.id)
+                                idType='RES_MGMT',
+                                resourceId=vnfc.computeResource.resourceId,
+                                vimConnectionId=(
+                                    vnfc.computeResource.vimConnectionId))
                             res_refs.append(res_ref)
 
+                # NOTE: It is meaningless if the target resource is less
+                # than 2, because it is intended to show the relationship
+                # between the resources.
+                if len(res_refs) < 2:
+                    break
+
                 plc_const = objects.PlacementConstraintV1(
                     affinityOrAntiAffinity=key,
                     scope=scope.upper(),
@@ -768,10 +840,9 @@ class VnfLcmDriverV2(object):
 
     def _modify_from_vnfd_prop(self, inst, vnfd_prop, attr):
         if vnfd_prop.get(attr):
-            setattr(inst, attr, vnfd_prop[attr])
-        elif inst.obj_attr_is_set(attr):
-            # set {} since attribute deletion is not supported.
-            setattr(inst, attr, {})
+            base = getattr(inst, attr) if inst.obj_attr_is_set(attr) else {}
+            setattr(inst, attr, inst_utils.json_merge_patch(
+                base, vnfd_prop[attr]))
 
     def _modify_from_req(self, inst, req, attr):
         if req.obj_attr_is_set(attr):
@@ -779,6 +850,21 @@ class VnfLcmDriverV2(object):
             setattr(inst, attr, inst_utils.json_merge_patch(
                 base, getattr(req, attr)))
 
+    def _modify_inst_vnfd_id(self, context, inst, vnfd):
+        pkg_info = self.nfvo_client.get_vnf_package_info_vnfd(
+            context, inst.vnfdId)
+
+        vnfd_prop = vnfd.get_vnfd_properties()
+
+        inst.vnfProvider = pkg_info.vnfProvider
+        inst.vnfProductName = pkg_info.vnfProductName
+        inst.vnfSoftwareVersion = pkg_info.vnfSoftwareVersion
+        inst.vnfdVersion = pkg_info.vnfdVersion
+
+        attrs = ['vnfConfigurableProperties', 'metadata', 'extensions']
+        for attr in attrs:
+            self._modify_from_vnfd_prop(inst, vnfd_prop, attr)
+
     def _merge_vim_connection_info(self, inst, req):
         # used by MODIFY_INFO and CHANGE_EXT_CONN
         # inst.vimConnectionInfo exists since req.vimConnectionInfo
@@ -804,26 +890,16 @@ class VnfLcmDriverV2(object):
             # NOTE: When vnfdId is changed, the values of attributes
             # in the VnfInstance needs update from new VNFD.
             inst.vnfdId = req.vnfdId
-
-            pkg_info = self.nfvo_client.get_vnf_package_info_vnfd(
-                context, inst.vnfdId)
-
             new_vnfd = self.nfvo_client.get_vnfd(context, inst.vnfdId)
-            new_vnfd_prop = new_vnfd.get_vnfd_properties()
-
-            inst.vnfProvider = pkg_info.vnfProvider
-            inst.vnfProductName = pkg_info.vnfProductName
-            inst.vnfSoftwareVersion = pkg_info.vnfSoftwareVersion
-            inst.vnfdVersion = pkg_info.vnfdVersion
-
-            attrs = ['vnfConfigurableProperties', 'metadata', 'extensions']
-            for attr in attrs:
-                self._modify_from_vnfd_prop(inst, new_vnfd_prop, attr)
+            self._modify_inst_vnfd_id(context, inst, new_vnfd)
 
         attrs = ['vnfConfigurableProperties', 'metadata', 'extensions']
         for attr in attrs:
             self._modify_from_req(inst, req, attr)
 
+        if inst.obj_attr_is_set('metadata'):
+            inst_utils.check_metadata_format(inst.metadata)
+
         if req.obj_attr_is_set('vimConnectionInfo'):
             self._merge_vim_connection_info(inst, req)
 
@@ -1122,11 +1198,19 @@ class VnfLcmDriverV2(object):
         elif vim_info.vimType == 'ETSINFV.HELM.V_3':
             driver = helm.Helm()
             driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'TERRAFORM.V1':
+            driver = terraform.Terraform()
+            driver.change_vnfpkg(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
 
         inst.vnfdId = grant_req.dstVnfdId
+        self._modify_inst_vnfd_id(context, inst, vnfd)
+
+        attrs = ['vnfConfigurableProperties', 'extensions']
+        for attr in attrs:
+            self._modify_from_req(inst, req, attr)
 
     def change_vnfpkg_rollback(
             self, context, lcmocc, inst, grant_req, grant, vnfd):
@@ -1143,6 +1227,9 @@ class VnfLcmDriverV2(object):
         elif vim_info.vimType == 'ETSINFV.HELM.V_3':
             driver = helm.Helm()
             driver.change_vnfpkg_rollback(req, inst, grant_req, grant, vnfd)
+        elif vim_info.vimType == 'TERRAFORM.V1':
+            driver = terraform.Terraform()
+            driver.change_vnfpkg_rollback(req, inst, grant_req, grant, vnfd)
         else:
             # should not occur
             raise sol_ex.SolException(sol_detail='not support vim type')
