@@ -27,7 +27,6 @@ from tacker import context
 from tacker import policy
 from tacker.sol_refactored.api import api_version
 from tacker.sol_refactored.api.policies.vnflcm_v2 import POLICY_NAME
-from tacker.sol_refactored.common import common_script_utils
 from tacker.sol_refactored.common import config
 from tacker.sol_refactored.common import exceptions as sol_ex
 from tacker.sol_refactored.common import lcm_op_occ_utils as lcmocc_utils
@@ -41,6 +40,7 @@ from tacker.sol_refactored.nfvo import nfvo_client
 from tacker.sol_refactored import objects
 from tacker.sol_refactored.objects.v2 import fields
 from tacker.tests.unit.db import base as db_base
+from tacker.tests import utils
 
 
 objects.register_all()
@@ -189,18 +189,18 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         # 'expected_status_code': success_status_code
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': success_status_code
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_all@all', 'VENDOR_all', 'TENANT_all'
@@ -209,7 +209,7 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_all@region_A', 'VENDOR_all', 'TENANT_all'
@@ -218,12 +218,12 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_vendor,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': success_status_code
         },
@@ -235,18 +235,18 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates_without_area,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
@@ -258,66 +258,66 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_B@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_B',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_A',
-                'TENANT_namespace_B'
+                'TENANT_tenant_B'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_area_A@region_A',
@@ -327,40 +327,40 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_area_vendor_tenant,
             'roles': [
                 'AREA_all@region_B',
                 'VENDOR_provider_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_vendor,
             'roles': [
                 'AREA_area_A@region_A',
                 'VENDOR_provider_B',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
-            'tenant': 'namespace_A',
+            'tenant': 'tenant_A',
             'rules': rule_vendor,
             'roles': [
                 'AREA_area_A@region_A',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
@@ -372,7 +372,7 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
             'roles': [
                 'AREA_area_B@region_A',
                 'VENDOR_provider_B',
-                'TENANT_namespace_A'
+                'TENANT_tenant_A'
             ],
             'expected_status_code': http_client.FORBIDDEN
         },
@@ -400,6 +400,39 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
     return test_data
 
 
+def get_test_data_policy_delete():
+    rules = {POLICY_NAME.format('delete'): "vendor:%(vendor)s"}
+    test_data = [
+        # 'expected_status_code': http_client.NO_CONTENT
+        {
+            'vnf_instance_updates': {'vnfProvider': 'provider_A'},
+            'rules': rules,
+            'roles': ['VENDOR_provider_A'],
+            'expected_status_code': http_client.NO_CONTENT
+        },
+        {
+            'vnf_instance_updates': {'vnfProvider': 'provider_A'},
+            'rules': rules,
+            'roles': ['VENDOR_all'],
+            'expected_status_code': http_client.NO_CONTENT
+        },
+        # 'expected_status_code': http_client.FORBIDDEN
+        {
+            'vnf_instance_updates': {'vnfProvider': 'provider_A'},
+            'rules': rules,
+            'roles': [],
+            'expected_status_code': http_client.FORBIDDEN
+        },
+        {
+            'vnf_instance_updates': {'vnfProvider': 'provider_A'},
+            'rules': rules,
+            'roles': ['VENDOR_provider_B'],
+            'expected_status_code': http_client.FORBIDDEN
+        },
+    ]
+    return test_data
+
+
 def make_vnf_instance(
         vim_type, instantiation_state, vendor, area=None, tenant=None):
     vim_connection_info = objects.VimConnectionInfo(
@@ -435,8 +468,8 @@ def make_vnf_instance(
             objects.VnfInstanceV2_InstantiatedVnfInfo(
                 flavourId='fake_flavour_id',
                 vnfState='STARTED',
-                metadata={'namespace': tenant}))
-        vnf_inst_fields.update({'metadata': {'namespace': tenant}})
+                metadata={'tenant': tenant}))
+        vnf_inst_fields.update({'metadata': {'tenant': tenant}})
 
     return vnf_instance
 
@@ -452,50 +485,59 @@ def get_test_data_policy_index():
 
     inst_1 = make_vnf_instance(
         'ETSINFV.OPENSTACK_KEYSTONE.V_3', 'INSTANTIATED', 'provider_A',
-        area='area_A@region_A')
+        area='area_A@region_A', tenant='tenant_A')
     # OK
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@all', 'VENDOR_all'],
+        'roles': ['AREA_all@all', 'VENDOR_all', 'TENANT_all'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@region_A', 'VENDOR_all'],
+        'roles': ['AREA_all@region_A', 'VENDOR_all', 'TENANT_all'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     # wrong region role
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@region_B', 'VENDOR_all'],
+        'roles': ['AREA_all@region_B', 'VENDOR_all', 'TENANT_all'],
         'expected_vnf_inst_ids': []
     })
     # wrong area role
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_B@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_area_B@region_A', 'VENDOR_provider_A', 'TENANT_all'],
         'expected_vnf_inst_ids': []
     })
     # wrong vendor role
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_B'],
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_B', 'TENANT_all'],
+        'expected_vnf_inst_ids': []
+    })
+    # wrong tenant role
+    test_data.append({
+        'vnf_instance_list': [inst_1],
+        'rules': rule_area_vendor_tenant,
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A', 'TENANT_B'],
         'expected_vnf_inst_ids': []
     })
     # without area
     inst_2 = make_vnf_instance(
-        'ETSINFV.OPENSTACK_KEYSTONE.V_3', 'INSTANTIATED', 'provider_A')
+        'ETSINFV.OPENSTACK_KEYSTONE.V_3', 'INSTANTIATED', 'provider_A',
+        tenant='tenant_A')
     test_data.append({
         'vnf_instance_list': [inst_2],
         'rules': rule_area_vendor_tenant,
@@ -504,7 +546,8 @@ def get_test_data_policy_index():
     })
     # NOT_INSTANTIATED
     inst_3 = make_vnf_instance(
-        'ETSINFV.OPENSTACK_KEYSTONE.V_3', 'NOT_INSTANTIATED', 'provider_A')
+        'ETSINFV.OPENSTACK_KEYSTONE.V_3', 'NOT_INSTANTIATED', 'provider_A',
+        tenant='tenant_A')
     test_data.append({
         'vnf_instance_list': [inst_3],
         'rules': rule_area_vendor_tenant,
@@ -707,7 +750,13 @@ class TestVnflcmV2(db_base.SqlTestCase):
             "vnfdId": vnfd_id,
             "vnfInstanceName": "test",
             "vnfInstanceDescription": "test",
-            "metadata": {"key": "value"}
+            "metadata": {
+                "key": "value",
+                "VDU_VNFc_mapping": {
+                    "VDU1": ["a-001", "a-002", "a-003"],
+                    "VDU2": ["b-001"]
+                }
+            }
         }
         result = self.controller.create(request=self.request, body=body)
         self.assertEqual(201, result.status)
@@ -1048,7 +1097,7 @@ class TestVnflcmV2(db_base.SqlTestCase):
         ex = self.assertRaises(sol_ex.InvalidSubscription,
             self.controller.subscription_create, request=self.request,
             body=body_1)
-        self.assertEqual("ParamsBasic must be specified.", ex.detail)
+        self.assertEqual("paramsBasic must be specified.", ex.detail)
 
         body_2 = {
             "callbackUri": "http://127.0.0.1:6789/notification",
@@ -1074,8 +1123,12 @@ class TestVnflcmV2(db_base.SqlTestCase):
         self.assertEqual("paramsOauth2ClientCert must be specified.",
                          ex.detail)
 
-    @mock.patch.object(common_script_utils, 'test_notification')
+    @mock.patch.object(subsc_utils, 'test_notification')
     def test_subscription_create_201(self, mock_test):
+        sample_cert = utils.test_sample(
+            "unit/sol_refactored/samples/sample_cert",
+            "notification_client_cert.pem")
+        CONF.v2_vnfm.notification_mtls_client_cert_file = sample_cert
         body_1 = {
             "callbackUri": "http://127.0.0.1:6789/notification",
             "authentication": {
@@ -1093,9 +1146,8 @@ class TestVnflcmV2(db_base.SqlTestCase):
                 "paramsOauth2ClientCert": {
                     "clientId": "test",
                     "certificateRef": {
-                        "type": "x5t#256",
-                        "value": "03c6e188d1fe5d3da8c9bc9a8dc531a2"
-                                 "b3ecf812b03aede9bec7ba1b410b6b64"
+                        "type": "x5t#S256",
+                        "value": "8Shbulz8zlFdKG-iMCUz5CCv0A7q0k6X7wL3NcZpshM"
                     },
                     "tokenEndpoint": "https://127.0.0.1/token"
                 }
@@ -1150,9 +1202,8 @@ class TestVnflcmV2(db_base.SqlTestCase):
                 "paramsOauth2ClientCert": {
                     "clientId": "test",
                     "certificateRef": {
-                        "type": "x5t#256",
-                        "value": "03c6e188d1fe5d3da8c9bc9a8dc531a2"
-                                 "b3ecf812b03aede9bec7ba1b410b6b64"
+                        "type": "x5t#S256",
+                        "value": "8Shbulz8zlFdKG-iMCUz5CCv0A7q0k6X7wL3NcZpshM"
                     },
                     "tokenEndpoint": "https://127.0.0.1/token"
                 }
@@ -1414,11 +1465,14 @@ class TestVnflcmV2(db_base.SqlTestCase):
     def test_lcm_op_occ_fail(self, mocked_send_lcmocc_notification):
         # prepare
         lcmocc = self._prepare_db_for_fail()
+        before_state_entered_time = lcmocc.stateEnteredTime
 
         op_state = []
+        op_state_entered_time = []
 
         def _store_state(context, lcmocc, inst, endpoint):
             op_state.append(lcmocc.operationState)
+            op_state_entered_time.append(lcmocc.stateEnteredTime)
 
         mocked_send_lcmocc_notification.side_effect = _store_state
 
@@ -1429,6 +1483,10 @@ class TestVnflcmV2(db_base.SqlTestCase):
         self.assertEqual(1, mocked_send_lcmocc_notification.call_count)
         self.assertEqual(fields.LcmOperationStateType.FAILED, op_state[0])
 
+        # check stateEnteredTime
+        self.assertNotEqual(before_state_entered_time,
+                            op_state_entered_time[0])
+
         # check grant_req and grant are deleted
         self.assertRaises(sol_ex.GrantRequestOrGrantNotFound,
             lcmocc_utils.get_grant_req_and_grant, self.context, lcmocc)
@@ -1829,7 +1887,7 @@ class TestVnflcmV2EnhancedPolicy(TestVnflcmV2):
                 objects.VnfInstanceV2_InstantiatedVnfInfo(
                     flavourId='fake_flavour_id',
                     vnfState='STARTED',
-                    metadata={'namespace': 'namespace_A'}))
+                    metadata={'tenant': 'tenant_A'}))
         inst.create(self.context)
         lcmocc.create(self.context)
 
@@ -1889,7 +1947,7 @@ class TestVnflcmV2EnhancedPolicy(TestVnflcmV2):
             'maxScaleLevels': max_scale_levels,
         }
         if tenant:
-            instantiated_vnf_info.update({'metadata': {'namespace': tenant}})
+            instantiated_vnf_info.update({'metadata': {'tenant': tenant}})
         inst.instantiatedVnfInfo = objects.VnfInstanceV2_InstantiatedVnfInfo(
             **instantiated_vnf_info
         )
@@ -1923,7 +1981,7 @@ class TestVnflcmV2EnhancedPolicy(TestVnflcmV2):
             ]
         )
         if tenant:
-            inst.instantiatedVnfInfo.metadata = {'namespace': tenant}
+            inst.instantiatedVnfInfo.metadata = {'tenant': tenant}
         if not vnf_inst_updates:
             inst.vimConnectionInfo = {
                 "vim1": objects.VimConnectionInfo.from_dict(
@@ -2092,17 +2150,15 @@ class TestVnflcmV2EnhancedPolicy(TestVnflcmV2):
             if expected_status_code != http_client.FORBIDDEN:
                 raise
 
-    @ddt.data(*get_test_data_policy_vnf_instantiated(
-        'delete', http_client.NO_CONTENT))
+    @ddt.data(*get_test_data_policy_delete())
     @ddt.unpack
     @mock.patch.object(nfvo_client.NfvoClient, 'send_inst_delete_notification')
     def test_delete_enhanced_policy(self, mock_delete, vnf_instance_updates,
-            tenant, rules, roles, expected_status_code):
+            rules, roles, expected_status_code):
         self._overwrite_policy(rules)
         inst_id, _ = self._create_inst_and_lcmocc('NOT_INSTANTIATED',
             fields.LcmOperationStateType.COMPLETED,
-            vnf_inst_updates=vnf_instance_updates,
-            tenant=tenant)
+            vnf_inst_updates=vnf_instance_updates)
         try:
             result = self.controller.delete(
                 self._fake_request(roles), id=inst_id)
@@ -2197,11 +2253,11 @@ class TestVnflcmV2EnhancedPolicy(TestVnflcmV2):
             ]
         )
         if tenant:
-            inst.instantiatedVnfInfo.metadata = {'namespace': tenant}
+            inst.instantiatedVnfInfo.metadata = {'tenant': tenant}
         # inst.instantiatedVnfInfo = objects.VnfInstanceV2_InstantiatedVnfInfo(
         #     flavourId='fake_flavour_id',
         #     vnfState='STARTED',
-        #     metadata={'namespace': tenant}
+        #     metadata={'tenant': tenant}
         # )
         inst.create(self.context)
 
