@@ -35,6 +35,7 @@ from tacker.objects import scale_vnf_request
 from tacker.objects.vim_connection import VimConnectionInfo
 from tacker.policies import vnf_lcm as vnf_lcm_policies
 from tacker.tests import constants
+from tacker.tests import utils as test_utils
 from tacker.tests import uuidsentinel
 from tacker import wsgi
 
@@ -716,19 +717,6 @@ def get_vnfd_dict(image_path=None):
     return vnfd_dict
 
 
-def get_dummy_vnf_instance():
-    connection_info = get_dummy_vim_connection_info()
-    return {'created_at': '', 'deleted': False, 'deleted_at': None,
-            'id': 'fake_id', 'instantiated_vnf_info': None,
-            'instantiation_state': 'NOT_INSTANTIATED',
-            'tenant_id': 'fake_tenant_id', 'updated_at': '',
-            'vim_connection_info': [connection_info],
-            'vnf_instance_description': 'VNF Description',
-            'vnf_instance_name': 'test', 'vnf_product_name': 'Sample VNF',
-            'vnf_provider': 'Company', 'vnf_software_version': '1.0',
-            'vnfd_id': 'fake_vnfd_id', 'vnfd_version': '1.0'}
-
-
 def get_dummy_vim_connection_info():
     return {'access_info': {
         'auth_url': 'fake/url',
@@ -843,8 +831,8 @@ def _get_vnf(**updates):
         'instance_id': uuidsentinel.instance_id,
         'attributes': {
             "scale_group": '{"scaleGroupDict" : {"SP1": {"maxLevel" : 3}}}',
-            "heat_template": os.path.dirname(__file__) +
-            "/../../etc/samples/hot_lcm_template.yaml"}}
+            "heat_template": test_utils.test_etc_sample(
+                "hot_lcm_template.yaml")}}
 
     if updates:
         vnf_data.update(**updates)
@@ -1986,65 +1974,95 @@ def get_test_data_policy_index():
     }
     test_data = []
     inst_1 = make_vnf_instance('openstack', 'provider_A',
-                               area='area_A@region_A')
+                               area='area_A@region_A', tenant='tenant_A')
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_all@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_1],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@all', 'VENDOR_all'],
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_tenant_A'],
+        'expected_vnf_inst_ids': [inst_1.id]
+    })
+    test_data.append({
+        'vnf_instance_list': [inst_1],
+        'rules': rule_area_vendor_tenant,
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_all'],
         'expected_vnf_inst_ids': [inst_1.id]
     })
     inst_2 = make_vnf_instance(
-        'openstack', 'provider_A', area='area_A@region_A',
+        'openstack', 'provider_A', area='area_A@region_A', tenant='tenant_A',
         instantiated_state=fields.VnfInstanceState.INSTANTIATED)
     test_data.append({
         'vnf_instance_list': [inst_2],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': [inst_2.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_2],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_all@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': [inst_2.id]
     })
     test_data.append({
         'vnf_instance_list': [inst_2],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@all', 'VENDOR_all'],
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_tenant_A'],
+        'expected_vnf_inst_ids': [inst_2.id]
+    })
+    test_data.append({
+        'vnf_instance_list': [inst_2],
+        'rules': rule_area_vendor_tenant,
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_all'],
         'expected_vnf_inst_ids': [inst_2.id]
     })
     inst_3 = make_vnf_instance(
-        'openstack', 'provider_A',
+        'openstack', 'provider_A', tenant='tenant_A',
         instantiated_state=fields.VnfInstanceState.INSTANTIATED)
     test_data.append({
         'vnf_instance_list': [inst_3],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': []
     })
     test_data.append({
         'vnf_instance_list': [inst_3],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@region_A', 'VENDOR_provider_A'],
+        'roles': ['AREA_all@region_A', 'VENDOR_provider_A',
+                  'TENANT_tenant_A'],
         'expected_vnf_inst_ids': []
     })
     test_data.append({
         'vnf_instance_list': [inst_3],
         'rules': rule_area_vendor_tenant,
-        'roles': ['AREA_all@all', 'VENDOR_all'],
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_tenant_A'],
+        'expected_vnf_inst_ids': []
+    })
+    test_data.append({
+        'vnf_instance_list': [inst_3],
+        'rules': rule_area_vendor_tenant,
+        'roles': ['AREA_all@all', 'VENDOR_all',
+                  'TENANT_all'],
         'expected_vnf_inst_ids': []
     })
     inst_4 = make_vnf_instance(
@@ -2130,6 +2148,50 @@ def get_test_data_policy_instantiate():
     return test_data
 
 
+def get_test_data_policy_delete():
+    test_data = [
+        # 'expected_status_code': http_client.NO_CONTENT
+        {
+            'vnf_instance_updates': {'vnf_provider': 'provider_A'},
+            'rules': {
+                "os_nfv_orchestration_api:vnf_instances:delete":
+                    "vendor:%(vendor)s"
+            },
+            'roles': ['VENDOR_provider_A'],
+            'expected_status_code': http_client.NO_CONTENT
+        },
+        {
+            'vnf_instance_updates': {'vnf_provider': 'provider_A'},
+            'rules': {
+                "os_nfv_orchestration_api:vnf_instances:delete":
+                    "vendor:%(vendor)s"
+            },
+            'roles': ['VENDOR_all'],
+            'expected_status_code': http_client.NO_CONTENT
+        },
+        # 'expected_status_code': http_client.FORBIDDEN
+        {
+            'vnf_instance_updates': {'vnf_provider': 'provider_A'},
+            'rules': {
+                "os_nfv_orchestration_api:vnf_instances:delete":
+                    "vendor:%(vendor)s"
+            },
+            'roles': [],
+            'expected_status_code': http_client.FORBIDDEN
+        },
+        {
+            'vnf_instance_updates': {'vnf_provider': 'provider_A'},
+            'rules': {
+                "os_nfv_orchestration_api:vnf_instances:delete":
+                    "vendor:%(vendor)s"
+            },
+            'roles': ['VENDOR_provider_B'],
+            'expected_status_code': http_client.FORBIDDEN
+        },
+    ]
+    return test_data
+
+
 def get_test_data_policy_vnf_not_instantiated(
         action, success_status_code=http_client.OK):
     # openstack
@@ -2168,7 +2230,8 @@ def get_test_data_policy_vnf_not_instantiated(
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': success_status_code
         },
         {
@@ -2194,7 +2257,8 @@ def get_test_data_policy_vnf_not_instantiated(
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_vendor,
-            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A'],
+            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': success_status_code
         },
         {
@@ -2209,6 +2273,13 @@ def get_test_data_policy_vnf_not_instantiated(
             'roles': ['AREA_all@region_A', 'VENDOR_all', 'TENANT_all'],
             'expected_status_code': success_status_code
         },
+        {
+            'vnf_instance_updates': vnf_instance_updates,
+            'rules': rule_area_vendor_tenant,
+            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_A',
+                      'TENANT_tenant_B'],
+            'expected_status_code': success_status_code
+        },
         # 'expected_status_code': http_client.FORBIDDEN
         {
             'vnf_instance_updates': vnf_instance_updates,
@@ -2231,37 +2302,49 @@ def get_test_data_policy_vnf_not_instantiated(
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_B'],
+            'roles': ['TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_area_B@region_A', 'VENDOR_provider_A'],
+            'roles': ['AREA_area_A@region_A', 'VENDOR_provider_B',
+                      'TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_all@region_B', 'VENDOR_provider_A'],
+            'roles': ['AREA_area_B@region_A', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_all', 'VENDOR_provider_A'],
+            'roles': ['AREA_all@region_B', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_all@', 'VENDOR_provider_A'],
+            'roles': ['AREA_all', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
+            'expected_status_code': http_client.FORBIDDEN
+        },
+        {
+            'vnf_instance_updates': vnf_instance_updates,
+            'rules': rule_area_vendor_tenant,
+            'roles': ['AREA_all@', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
         {
             'vnf_instance_updates': vnf_instance_updates_with_wrong_area,
             'rules': rule_area_vendor_tenant,
-            'roles': ['AREA_all@', 'VENDOR_provider_A'],
+            'roles': ['AREA_all@', 'VENDOR_provider_A',
+                      'TENANT_tenant_A'],
             'expected_status_code': http_client.FORBIDDEN
         },
     ]
@@ -2283,7 +2366,7 @@ def make_vnf_instance_updates(
         'vim_connection_info': [vim_connection_info]
     }
     if tenant:
-        vnf_instance_updates.update({'vnf_metadata': {"namespace": tenant}})
+        vnf_instance_updates.update({'vnf_metadata': {"tenant": tenant}})
 
     return vnf_instance_updates
 
@@ -2298,13 +2381,15 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
     test_data = []
     # openstack
     vnf_instance_updates_1 = make_vnf_instance_updates(
-        'openstack', 'provider_A', area='area_A@region_A')
+        'openstack', 'provider_A', area='area_A@region_A',
+        tenant='tenant_A')
     test_data.append({
         'vnf_instance_updates': vnf_instance_updates_1,
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_area_A@region_A',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': success_status_code
     })
@@ -2313,7 +2398,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_all@region_A',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': success_status_code
     })
@@ -2322,7 +2408,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_all@all',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': success_status_code
     })
@@ -2331,7 +2418,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_all@all',
-            'VENDOR_all'
+            'VENDOR_all',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': success_status_code
     })
@@ -2340,7 +2428,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_all@all',
-            'VENDOR_all'
+            'VENDOR_all',
+            'TENANT_all'
         ],
         'expected_status_code': success_status_code
     })
@@ -2350,7 +2439,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_area_B@region_A',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': http_client.FORBIDDEN
     })
@@ -2360,7 +2450,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_area_A@region_B',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': http_client.FORBIDDEN
     })
@@ -2370,11 +2461,22 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_area_A@region_A',
-            'VENDOR_provider_B'
+            'VENDOR_provider_B',
+            'TENANT_tenant_A'
+        ],
+        'expected_status_code': http_client.FORBIDDEN
+    })
+    # wrong tenant role
+    test_data.append({
+        'vnf_instance_updates': vnf_instance_updates_1,
+        'rules': rule_area_vendor_tenant,
+        'roles': [
+            'AREA_area_A@region_A',
+            'VENDOR_provider_A',
+            'TENANT_tenant_B'
         ],
         'expected_status_code': http_client.FORBIDDEN
     })
-
     vnf_instance_updates_2 = make_vnf_instance_updates(
         'openstack', 'provider_A')
     test_data.append({
@@ -2382,7 +2484,8 @@ def get_test_data_policy_vnf_instantiated(action, success_status_code):
         'rules': rule_area_vendor_tenant,
         'roles': [
             'AREA_area_A@region_A',
-            'VENDOR_provider_A'
+            'VENDOR_provider_A',
+            'TENANT_tenant_A'
         ],
         'expected_status_code': http_client.FORBIDDEN
     })
